import {
  Loader, Metadata, IEventParam, I18n, dayjs as moment, Config,
} from '@zeedhi/core';
import {
  Card, IterableComponentRender, Tabs, Tab, LoadingService, Form, Text, Button, Select,
} from '@zeedhi/common';
import { BottomSheet } from '@zeedhi/zd-bottom-sheet';
import { ScmMask } from '@/config';

export class AFController {
  private utilities = Loader.getInstance('Utilities');

  public showAfDetail = false;

  private periodo = {
    DTINICIAL: '',
    DTFINAL: '',
  };

  public get setFilterAFs() {
    return {
      CDFILIAL: localStorage.getItem('CDFILIALLOGADA') || '',
      DTINICIAL: this.periodo.DTINICIAL || '',
      DTFINAL: this.periodo.DTFINAL || '',
    };
  }

  get isVisibleDetailList() {
    return !this.showAfDetail;
  }

  public setValueQtde({ component }: IEventParam<Text>) {
    const value = component.name.split('-').reverse()[0];
    // const mask = {
    //   decimalPlates: 5,
    // };
    const mask = Config.masks.productMask as ScmMask;
    const maskNumber = this.utilities.getMaskNumber(value, mask);
    component.text = `${I18n.translate('Qtde: ')} ${maskNumber}`;
  }

  public async clickCardAf({ component }: IEventParam<Card>) {
    LoadingService.show(' ');
    const mask = {
      decimalPlates: 5,
    };
    this.showAfDetail = true;
    const tabsAfs = Metadata.getInstance<Tabs>('tabsAF');
    const afsRender = Metadata.getInstance<IterableComponentRender>('iterableCardsAF');
    tabsAfs.tabs = afsRender.datasource.data.map((row) => new Tab({
      name: `${row.CDPRODUTO}`,
      tabTitle: `${this.utilities.getValueWithMaskProduto(row.CDPRODUTO)} - ${row.NMPRODUTO}`,
      children: [
        {
          allowDuplicate: true,
          name: `tagTab-${row.CDPRODUTO}`,
          component: 'ZdTag',
          tag: 'div',
          isVisible: false,
        },
        {
          name: `formDetailAf-${row.CDPRODUTO}`,
          component: 'ZdForm',
          children: [
            {
              allowDuplicate: true,
              name: 'CDPRODUTO',
              component: 'ZdText',
              readonly: true,
              text: `${I18n.translate('Produto: ')} ${this.utilities.getValueWithMaskProduto(row.CDPRODUTO)} ${' | '}
              ${row.NMPRODUTO}`,
              grid: {
                cols: 12,
              },
            },
            {
              allowDuplicate: true,
              name: 'SGUNIDADE',
              component: 'ZdText',
              readonly: true,
              text: `${I18n.translate('Unidade: ')} ${row.SGUNIDADE}`,
              grid: {
                cols: 12,
              },
            },
            {
              allowDuplicate: true,
              name: 'QTPROGENTAF',
              component: 'ZdText',
              readonly: true,
              text: `${I18n.translate('Qtde: ')} ${this.utilities.getMaskNumber(row.QTPROGENTAF || '', mask)}`,
              grid: {
                cols: 12,
              },
            },
            {
              allowDuplicate: true,
              name: 'NMMARCA',
              component: 'ZdText',
              readonly: true,
              text: `${I18n.translate('Marca: ')} ${row.NMMARCA}`,
              grid: {
                cols: 12,
              },
            },
            {
              allowDuplicate: true,
              name: 'DTPROGENTAF',
              component: 'ZdText',
              readonly: true,
              text: `${I18n.translate('Dt Entrega: ')} ${this.utilities.formatDate(row.DTPROGENTAF)}`,
              grid: {
                cols: 12,
              },
            },
            {
              allowDuplicate: true,
              name: 'VRPREUNIAF',
              component: 'ZdText',
              readonly: true,
              text: `${I18n.translate('Valor Unitário: ')} ${this.utilities.getMaskNumber(row.VRPREUNIAF || '', mask)}`,
              grid: {
                cols: 12,
              },
            },
            {
              allowDuplicate: true,
              name: 'VRTOTALITEM',
              component: 'ZdText',
              readonly: true,
              text: `${I18n.translate('Valor Total: ')} ${this.utilities.getMaskNumber(row.VRTOTALITEM || '', mask)}`,
              grid: {
                cols: 12,
              },
            },
            {
              allowDuplicate: true,
              name: 'NRAUTOFORN',
              component: 'ZdText',
              readonly: true,
              text: `${I18n.translate('AF: ')} ${row.NRAUTOFORN}`,
              grid: {
                cols: 12,
              },
            },
            {
              allowDuplicate: true,
              name: 'CDFILLANC',
              component: 'ZdText',
              readonly: true,
              text: `${I18n.translate('Unidade Lançamento: ')} ${row.CDFILLANC}`,
              grid: {
                cols: 12,
              },
            },
            {
              allowDuplicate: true,
              name: 'CDFILDEST',
              component: 'ZdText',
              readonly: true,
              text: `${I18n.translate('Unidade Destino: ')} ${row.CDFILDEST}`,
              grid: {
                cols: 12,
              },
            },
          ],
        },
      ],
    }));
    const index = component.name;
    tabsAfs.activeTab = await tabsAfs.tabs.findIndex((tab) => tab.children[0].name === `tagTab-${index}`);
    LoadingService.hide();
  }

  public clickBackAFs() {
    this.showAfDetail = false;
    // this.showInventarioDetail = true;
    const afs = Metadata.getInstance<IterableComponentRender>('iterableCardsAF');
    afs.reload();
  }

  public clickFilterAF() {
    const bottomSheet = Metadata.getInstance<BottomSheet>('filterSheet');
    const filial = Metadata.getInstance<Select>('CDFILIALLOGADA');
    if (filial.value) {
      bottomSheet.isVisible = true;
      bottomSheet.isOpen = true;
    }
  }

  public ordenaAF({ component }: IEventParam<Button>) {
    const { name } = component;
    const af = Metadata.getInstance<IterableComponentRender>('iterableCardsAF');
    const ds = af.datasource;
    const order = ds.getOrderByColumn(name);
    if (order) {
      if (order === 'desc') ds.addOrder(name, 'asc');
      else ds.addOrder(name, 'desc');
    } else ds.addOrder(name, 'desc');
  }

  public openFilter() {
    const btn = Metadata.getInstance<Button>('filterAFButton');
    btn.click();
  }

  public clickCloseFilterBtn() {
    const bottomSheet = Metadata.getInstance<BottomSheet>('filterSheet');
    bottomSheet.isVisible = false;
    bottomSheet.isOpen = false;
  }

  public setPeriodoAF({ component }: IEventParam<Button>) {
    const form = component.parent as Form;
    if (form.validate() && form.value.PERIODO.length === 2) {
      this.periodo.DTINICIAL = moment(form.value.PERIODO[0], 'YYYY-MM-DD').format('DD/MM/YYYY');
      this.periodo.DTFINAL = moment(form.value.PERIODO[1], 'YYYY-MM-DD').format('DD/MM/YYYY');
      const af = Metadata.getInstance<IterableComponentRender>('iterableCardsAF');
      af.datasource.search = form.value.search;
      af.datasource.order = [];
      af.reload();
      const bottomSheet = Metadata.getInstance<BottomSheet>('filterSheet');
      bottomSheet.isVisible = false;
      bottomSheet.isOpen = false;
    }
  }
}
