import {
  Loader, IEventParam, Metadata, dayjs as moment, I18n, IDictionary, DatasourceFactory, Config, Router,
} from '@zeedhi/core';
import {
  Card, IterableComponentRender, Tabs, Tab, Increment, Button, LoadingService, Form, DialogService, Text, Select,
} from '@zeedhi/common';
import { BottomSheet } from '@zeedhi/zd-bottom-sheet';
import { ScmMask } from '@/config';
import { HttpCRUDTranslator } from '@/http-crud-translator/http-crud-translator';

export class InventariosController {
  private utilities = Loader.getInstance('Utilities');

  private appController = Loader.getInstance('AppController');

  public showInventarioDetail = false;

  public showEditProdutosInventario = false;

  public showAddInventario = false;

  public showAddProduto = false;

  public inventarioDescription = '';

  public inventarioSelected = '';

  public notPossibleDates: string[] = [];

  private periodo = {
    DTINICIAL: '',
    DTFINAL: '',
  };

  public openAddInventario() {
    const form = Metadata.getInstance<Form>('addInventarioForm');
    form.clearForm();
    this.showAddInventario = true;
    this.setNotPossibleDates();
  }

  get isVisibleInventariosList() {
    return !this.showInventarioDetail && !this.showEditProdutosInventario && !this.showAddInventario && !this.showAddProduto;
  }

  public get setFilterInventarios() {
    return {
      CDFILIAL: localStorage.getItem('CDFILIALLOGADA') || '',
      DTINICIAL: this.periodo.DTINICIAL || '',
      DTFINAL: this.periodo.DTFINAL || '',
    };
  }

  public get setFilterProdutosInventario() {
    return {
      CDFILIAL: localStorage.getItem('CDFILIALLOGADA') || '',
      NRINVENTRETI: this.inventarioSelected || '',
    };
  }

  public clickBackDetailInventario() {
    this.showInventarioDetail = false;
    this.inventarioSelected = '';
  }

  public clickBackEditProdutos() {
    this.showEditProdutosInventario = false;
    this.showInventarioDetail = true;
    const produtos = Metadata.getInstance<IterableComponentRender>('iterableCardsProdutosInventario');
    produtos.reload();
  }

  public clickBackAddInventario() {
    this.showAddInventario = false;
  }

  public clickBackAddProduto() {
    this.showAddProduto = false;
    this.showInventarioDetail = true;
    const produtos = Metadata.getInstance<IterableComponentRender>('iterableCardsProdutosInventario');
    produtos.reload();
  }

  public clickCardInventario({ component }: IEventParam<Card>) {
    this.showInventarioDetail = true;
    const split = component.name.split('_');
    const numero = split[1];
    const nome = split[2];
    const data = split[3];
    this.inventarioSelected = numero;
    this.inventarioDescription = `${data} - ${nome}`;
    const produtosRender = Metadata.getInstance<IterableComponentRender>('iterableCardsProdutosInventario');
    produtosRender.reload();
  }

  public async clickCardProdutoInventario({ component }: IEventParam<Card>) {
    LoadingService.show(' ');
    this.showEditProdutosInventario = true;
    this.showInventarioDetail = false;
    const tabsProdutos = Metadata.getInstance<Tabs>('tabsProdutos');
    const produtosRender = Metadata.getInstance<IterableComponentRender>('iterableCardsProdutosInventario');
    if (tabsProdutos.tabs.length > 0) {
      // destrói os elementos das tabs que vão ser substituídas para não acontecer conflito de elementos
      await Promise.all(tabsProdutos.tabs.map(async (tab: IDictionary) => {
        await tab.children.map(async (child: IDictionary) => {
          await tab.removeChild(child.name);
        });
      })).then(() => {
        tabsProdutos.tabs = [];
        tabsProdutos.tabs = produtosRender.datasource.data.map((row) => this.newTabs(row));
      });
    } else tabsProdutos.tabs = produtosRender.datasource.data.map((row) => this.newTabs(row));
    const index = component.name.split('-')[1];
    tabsProdutos.activeTab = await tabsProdutos.tabs.findIndex((tab) => tab.children[0].name === `tagTab-${index}`);
    LoadingService.hide();
  }

  private newTabs(row: IDictionary) {
    return new Tab({
      name: `${row.CDPRODUTO}`,
      tabTitle: `${this.utilities.getValueWithMaskProduto(row.CDPRODUTO)} - ${row.NMPRODUTO}`,
      children: [
        {
          allowDuplicate: true,
          name: `tagTab-${row.NRINVENTRETI}_${row.CDPRODUTO}`,
          component: 'ZdTag',
          tag: 'div',
          children: [
            {
              name: `formEditProduct-${row.NRINVENTRETI}_${row.CDPRODUTO}`,
              component: 'ZdForm',
              children: [
                {
                  allowDuplicate: true,
                  name: 'CDARVPROD',
                  component: 'ZdText',
                  // label: 'Código',
                  // value: `${row.CDARVPROD}`,
                  // mask: '{{Utilities.getMaskProduto}}',
                  text: `${I18n.translate('Produto: ')} ${this.utilities.getValueWithMaskProduto(row.CDARVPROD)} ${' | '}
                  ${row.NMPRODUTO}`,
                  grid: {
                    cols: 12,
                  },
                },
                {
                  allowDuplicate: true,
                  name: 'CDPRODUTO',
                  isVisible: false,
                  component: 'ZdTextInput',
                  readonly: true,
                  label: 'Código',
                  value: `${row.CDPRODUTO}`,
                  mask: '{{Utilities.getMaskProduto}}',
                  grid: {
                    cols: 12,
                  },
                },
                {
                  allowDuplicate: true,
                  // name: `nome-${row.NRINVENTRETI}_${row.CDPRODUTO}`,
                  name: 'NMPRODUTO',
                  component: 'ZdTextInput',
                  isVisible: false,
                  readonly: true,
                  label: 'Nome',
                  value: `${row.NMPRODUTO}`,
                  grid: {
                    cols: 12,
                  },
                },
                {
                  allowDuplicate: true,
                  // name: `quantidade-${row.NRINVENTRETI}_${row.CDPRODUTO}`,
                  name: 'QTITINVENTRETI',
                  component: 'ZdIncrement',
                  label: 'Quantidade',
                  clearable: false,
                  step: 1,
                  events: {
                    change: '{{InventariosController.changeQuantProduct}}',
                  },
                  validations: {
                    required: {},
                    min: {
                      limit: 0.1,
                    },
                  },
                  value: row.QTITINVENTRETI,
                  grid: {
                    cols: 12,
                  },
                },
                {
                  allowDuplicate: true,
                  name: 'NRITINVENTRETI',
                  component: 'ZdTextInput',
                  isVisible: false,
                  value: `${row.NRITINVENTRETI}`,
                },
              ],
            },
            {
              name: `salvarEditProdutoBtn-${row.NRINVENTRETI}_${row.CDPRODUTO}`,
              component: 'ZdButton',
              label: 'Salvar',
              block: true,
              fixed: true,
              bottom: true,
              disabled: true,
              events: {
                click: '{{InventariosController.salvarEditProduto}}',
              },
              cssStyle: {
                right: '0px',
              },
            },
          ],
        },
      ],
    });
  }

  public setNotPossibleDates() {
    const inventarios = Metadata.getInstance<IterableComponentRender>('iterableCardsInventarios');
    const cloneConfig = inventarios.datasource.clone();
    const cloneDs = DatasourceFactory.factory(cloneConfig);
    cloneDs.loadAll = true;
    cloneDs.events = {};
    cloneDs.get().then((result: IDictionary) => {
      const inventariosDates = result.map((row: IDictionary) => {
        const dateRow = moment(row.DTINVENTRETI, 'DD/MM/YYYY');
        return dateRow.format('YYYY-MM-DD');
      });
      this.notPossibleDates = inventariosDates;
    });
  }

  public allowedDatesAddInventario(date: string) {
    let valid = true;
    const now = moment().subtract(1, 'day');
    const dateVerify = moment(date, Config.dateFormat);
    const dtFinal = moment(localStorage.getItem('DATA_FINAL'), Config.dateFormat);
    if (dateVerify >= now) {
      const inventarios = Loader.getInstance('InventariosController');
      if (inventarios.notPossibleDates.includes(dateVerify.format('DD/MM/YYYY'))) valid = false;
      if (dateVerify > dtFinal) valid = false;
    } else valid = false;
    return valid;
  }

  public salvarInventario() {
    const form = Metadata.getInstance<Form>('addInventarioForm');
    if (form.validate()) {
      const { value } = form;
      value.CDFILIAL = localStorage.getItem('CDFILIALLOGADA') || '';
      value.DTINVENTRETI = moment(value.DTINVENTRETI, Config.dateFormat).format('DD/MM/YYYY');
      LoadingService.show(' ');
      HttpCRUDTranslator.post('/inventarios/save', { requestType: 'DataSet', dataset: [value] })
        .then((response: IDictionary) => {
          if (!response.data.error) {
            this.utilities.showAlert('successSave', 'SUCCESS_SAVE', 'green');
            const inventarios = Metadata.getInstance<IterableComponentRender>('iterableCardsInventarios');
            inventarios.reload();
            this.clickBackAddInventario();
          } else {
            const mensagem = response.data.error;
            this.utilities.showAlert('errorSave', I18n.translate(mensagem), 'error');
          }
        }).catch((error: string) => {
          this.utilities.showAlert('alert', `${error}`, 'error');
        }).finally(() => {
          LoadingService.hide();
        });
    }
  }

  public deleteInventario({ component }: IEventParam<Button>) {
    const nrInvent = component.name.split('-')[1];
    const dataset = [
      {
        CDFILIAL: localStorage.getItem('CDFILIALLOGADA') || '',
        NRINVENTRETI: nrInvent,
        KEY: `${localStorage.getItem('CDFILIALLOGADA')}${nrInvent}`,
      },
    ];
    LoadingService.show(' ');
    HttpCRUDTranslator.post('/inventarios/delete', { requestType: 'DataSet', dataset }, {})
      .then((response: IDictionary) => {
        if (!response.data.error) {
          // this.utilities.showAlert('successSave', 'SUCCESS_SAVE', 'green', false);
          const inventarios = Metadata.getInstance<IterableComponentRender>('iterableCardsInventarios');
          inventarios.reload();
        } else {
          const mensagem = response.data.error;
          this.utilities.showAlert('errorSave', I18n.translate(mensagem), 'error');
        }
      }).catch((error: string) => {
        this.utilities.showAlert('alert', `${error}`, 'error');
      }).finally(() => {
        DialogService.hide();
        LoadingService.hide();
      });
  }

  public changeQuantProduct({ component }: IEventParam<Increment>) {
    const size = component.parent?.parent?.children?.length || 0;
    const btn = component.parent?.parent?.children[size - 1] as Button;
    btn.disabled = false;
  }

  public adicionarProdutoInventario() {
    const form = Metadata.getInstance<Form>('addProdutoInventarioForm');
    form.clearForm();
    this.showInventarioDetail = false;
    this.showAddProduto = true;
  }

  public get setFilterProdutos() {
    return {
      CDFILIAL: [localStorage.getItem('CDFILIALLOGADA')] || [],
      CDGRUPPROD: localStorage.getItem('CDGRUPPROD') || 'T',
    };
  }

  public clickDeleteInventario({ component, event }: IEventParam<Button>) {
    (event as IDictionary).stopPropagation();
    DialogService.show({
      name: 'deleteInventario',
      type: 'warning',
      title: I18n.translate('scm::ATTENTION'),
      text: I18n.translate('scm::CONFIRM_DELETE_INVENTORY'),
      buttons: [
        {
          name: 'yes',
          component: 'ZdButton',
          label: I18n.translate('scm::YES'),
          events: { click: () => this.deleteInventario({ component }) },
          outline: true,
        },
        {
          name: 'no',
          component: 'ZdButton',
          label: I18n.translate('scm::NO'),
          events: { click: () => DialogService.hide() },
        },
      ],
    });
  }

  public clickDeleteProdutoInventario({ component, event }: IEventParam<Button>) {
    (event as IDictionary).stopPropagation();
    DialogService.show({
      name: 'deleteProdutoInventario',
      type: 'warning',
      title: I18n.translate('scm::ATTENTION'),
      text: I18n.translate('scm::CONFIRM_DELETE_REGISTER'),
      buttons: [
        {
          name: 'yes',
          component: 'ZdButton',
          label: I18n.translate('scm::YES'),
          events: { click: () => this.deleteProdutoInventario({ component }) },
          outline: true,
        },
        {
          name: 'no',
          component: 'ZdButton',
          label: I18n.translate('scm::NO'),
          events: { click: () => DialogService.hide() },
        },
      ],
    });
  }

  public deleteProdutoInventario({ component }: IDictionary) {
    const nrItInvent = component.name.split('-')[1].split('_')[1];
    const cdroduto = component.name.split('-')[1].split('_')[2];
    const dataset = [
      {
        CDFILIAL: localStorage.getItem('CDFILIALLOGADA') || '',
        NRINVENTRETI: this.inventarioSelected,
        NRITINVENTRETI: nrItInvent,
        CDPRODUTO: cdroduto,
        KEY: `${localStorage.getItem('CDFILIALLOGADA')}${this.inventarioSelected}${nrItInvent}`,
      },
    ];
    LoadingService.show(' ');
    HttpCRUDTranslator.post(
      '/produtosInventario/delete',
      { requestType: 'DataSet', dataset },
      {},
    ).then((response: IDictionary) => {
      if (!response.data.error) {
        // this.utilities.showAlert('successSave', 'SUCCESS_SAVE', 'green', false);
        const produtos = Metadata.getInstance<IterableComponentRender>('iterableCardsProdutosInventario');
        produtos.reload();
      } else {
        const mensagem = response.data.error;
        this.utilities.showAlert('errorSave', I18n.translate(mensagem), 'error');
      }
    }).catch((error: string) => {
      this.utilities.showAlert('alert', `${error}`, 'error');
    }).finally(() => {
      DialogService.hide();
      LoadingService.hide();
    });
  }

  public salvarProdutoInventario() {
    const form = Metadata.getInstance<Form>('addProdutoInventarioForm');
    if (form.validate()) {
      const dataset = [
        {
          CDFILIAL: localStorage.getItem('CDFILIALLOGADA') || '',
          NRINVENTRETI: this.inventarioSelected,
          new: true,
          ...form.value,
        },
      ];
      LoadingService.show(' ');
      HttpCRUDTranslator.post('/produtosInventario/save', { requestType: 'DataSet', dataset })
        .then((response: IDictionary) => {
          if (!response.data.error) {
            this.utilities.showAlert('successSave', 'SUCCESS_SAVE', 'green');
            form.clearForm();
          } else {
            const mensagem = response.data.error;
            this.utilities.showAlert('errorSave', I18n.translate(mensagem), 'error');
          }
        }).catch((error: string) => {
          this.utilities.showAlert('alert', error, 'error');
        }).finally(() => {
          LoadingService.hide();
        });
    }
  }

  public salvarEditProduto({ component }: IEventParam<Button>) {
    // const form = Metadata.getInstance<Form>('formEditProduct');
    const form = component.parent?.getChildInstance<Form>(component.parent?.children[0].name);
    if (form?.validate()) {
      const dataset = [
        {
          CDFILIAL: localStorage.getItem('CDFILIALLOGADA') || '',
          NRINVENTRETI: this.inventarioSelected,
          new: false,
          ...form?.value,
          KEY: `${localStorage.getItem('CDFILIALLOGADA')}${this.inventarioSelected}${form?.value.NRITINVENTRETI}`,
        },
      ];
      LoadingService.show(' ');
      HttpCRUDTranslator.post('/produtosInventario/update', { requestType: 'DataSet', dataset })
        .then((response: IDictionary) => {
          if (!response.data.error) {
            this.utilities.showAlert('successSave', 'SUCCESS_SAVE', 'green');
            component.disabled = true;
            const tabsProdutos = Metadata.getInstance<Tabs>('tabsProdutos');
            tabsProdutos.nextTab();
          } else {
            const mensagem = response.data.error;
            this.utilities.showAlert('errorSave', I18n.translate(mensagem), 'error');
          }
        }).catch((error: string) => {
          this.utilities.showAlert('alert', error, 'error');
        }).finally(() => {
          LoadingService.hide();
        });
    }
  }

  public setValueQtde({ component }: IEventParam<Text>) {
    const value = component.name.split('-')[1].split('_')[2] || '';
    const mask = Config.masks.productMask as ScmMask;
    const maskNumber = this.utilities.getMaskNumber(value, mask);
    component.text = `${maskNumber}`;
  }

  public setPeriodoInventarios({ component }: IEventParam<Button>) {
    const form = component.parent as Form;
    if (form.validate() && form.value.PERIODO.length === 2) {
      this.periodo.DTINICIAL = moment(form.value.PERIODO[0], 'YYYY-MM-DD').format('DD/MM/YYYY');
      this.periodo.DTFINAL = moment(form.value.PERIODO[1], 'YYYY-MM-DD').format('DD/MM/YYYY');
      const inventarios = Metadata.getInstance<IterableComponentRender>('iterableCardsInventarios');
      // inventarios.datasource.setSearch(form.value.search);
      inventarios.datasource.search = form.value.search;
      // inventarios.datasource.clearOrder();
      inventarios.datasource.order = [];
      inventarios.reload();
      const bottomSheet = Metadata.getInstance<BottomSheet>('filterSheet');
      bottomSheet.isVisible = false;
      bottomSheet.isOpen = false;
    }
  }

  public clickFilterInventarios() {
    const bottomSheet = Metadata.getInstance<BottomSheet>('filterSheet');
    const filial = Metadata.getInstance<Select>('CDFILIALLOGADA');
    if (filial.value && filial.value.CDFILCENTPME && filial.value.CDFILIAL !== filial.value.CDFILCENTPME) {
      bottomSheet.isVisible = true;
      bottomSheet.isOpen = true;
    }
  }

  public clickCloseFilterBtn() {
    const bottomSheet = Metadata.getInstance<BottomSheet>('filterSheet');
    bottomSheet.isVisible = false;
    bottomSheet.isOpen = false;
  }

  public openFilter() {
    const btn = Metadata.getInstance<Button>('filterInventariosButton');
    btn.click();
  }

  public closeProdutosFilterBtn() {
    const bottomSheet = Metadata.getInstance<BottomSheet>('produtosfilterSheet');
    bottomSheet.isVisible = false;
    bottomSheet.isOpen = false;
  }

  public clickFilterProdutos() {
    const bottomSheet = Metadata.getInstance<BottomSheet>('produtosfilterSheet');
    bottomSheet.isVisible = true;
    bottomSheet.isOpen = true;
  }

  public setSearchProdutosFilter({ component }: IEventParam<Button>) {
    const form = component.parent as Form;
    if (form.validate()) {
      const produtos = Metadata.getInstance<IterableComponentRender>('iterableCardsProdutosInventario');
      produtos.datasource.search = form.value.searchProdutos;
      produtos.datasource.order = [];
      produtos.reload();
      const bottomSheet = Metadata.getInstance<BottomSheet>('produtosfilterSheet');
      bottomSheet.isVisible = false;
      bottomSheet.isOpen = false;
    }
  }

  public confirmCopyProducts() {
    DialogService.show({
      name: 'confirmCopyProducts',
      type: 'info',
      title: I18n.translate('scm::COPY_LAST_INVENTORY'),
      text: I18n.translate('scm::COPY_PRODUCTS_INVENTORY'),
      buttons: [
        {
          name: 'yes',
          component: 'ZdButton',
          label: I18n.translate('scm::YES'),
          events: {
            click: () => {
              this.copiarUltimoInvertario();
            },
          },
          outline: true,
        },
        {
          name: 'no',
          component: 'ZdButton',
          label: I18n.translate('scm::NO'),
          events: { click: () => DialogService.hide() },
        },
      ],
    });
  }

  public copiarUltimoInvertario() {
    const ds = DatasourceFactory.factory({
      uniqueKey: 'NRINVENTRETI',
      type: 'rest',
      route: '/ultimoInventario',
      filter: {
        CDFILIAL: localStorage.getItem('CDFILIALLOGADA') || '',
      },
    });
    DialogService.hide();
    LoadingService.show(' ');
    ds.get().then((result: IDictionary) => {
      if (result.length > 0) {
        const produtos = Metadata.getInstance<IterableComponentRender>('iterableCardsProdutosInventario');
        const dsConfig = produtos.datasource.clone();
        dsConfig.allData = true;
        const clonedDs = DatasourceFactory.factory(dsConfig);
        clonedDs.get().then((products: IDictionary) => {
          if (products.length > 0) {
            HttpCRUDTranslator.post('/produtosInventario/delete', { requestType: 'DataSet', dataset: products }, {})
              .then((response: IDictionary) => {
                if (!response.data.error) {
                  this.salvaProdutosUltimoInventario(result);
                } else {
                  const mensagem = response.data.error;
                  this.utilities.showAlert('errorSave', I18n.translate(mensagem), 'error');
                }
              }).catch((error: string) => {
                this.utilities.showAlert('alert', `${error}`, 'error');
                LoadingService.hide();
              });
          } else {
            this.salvaProdutosUltimoInventario(result);
          }
        });
      } else {
        LoadingService.hide();
        DialogService.show({
          name: 'errorCopyInventory',
          type: 'warning',
          title: I18n.translate('scm::ATTENTION'),
          text: I18n.translate('scm::NO_INVENTORY_REGISTRED'),
          buttons: [
            {
              name: 'ok',
              component: 'ZdButton',
              label: I18n.translate('scm::OK'),
              events: { click: () => DialogService.hide() },
            },
          ],
        });
      }
    });
  }

  public async salvaProdutosUltimoInventario(products: IDictionary) {
    await products.forEach((product: IDictionary) => {
      product.QTITINVENTRETI = 0;
      product.NRINVENTRETI = this.inventarioSelected || '';
    });
    HttpCRUDTranslator.post('/produtosInventario/save', { requestType: 'DataSet', dataset: products })
      .then((response: IDictionary) => {
        if (!response.data.error) {
          const produtos = Metadata.getInstance<IterableComponentRender>('iterableCardsProdutosInventario');
          produtos.reload();
        } else {
          const mensagem = response.data.error;
          this.utilities.showAlert('errorSave', I18n.translate(mensagem), 'error');
        }
      }).catch((error: string) => {
        this.utilities.showAlert('alert', `${error}`, 'error');
      }).finally(() => {
        LoadingService.hide();
      });
  }

  public checkAccess() {
    const filial = Metadata.getInstance<Select>('CDFILIALLOGADA');
    if (filial.value) {
      let text = '';
      if (!filial.value.CDFILCENTPME) {
        text = 'scm::BLOCK_ACCESS_INVENTORY_1';
        DialogService.show({
          name: 'blockAccessInventory',
          type: 'error',
          title: I18n.translate('scm::ATTENTION'),
          text: I18n.translate(text),
          buttons: [
            {
              name: 'ok',
              component: 'ZdButton',
              label: I18n.translate('scm::OK'),
              events: {
                click: () => {
                  DialogService.hide();
                  Router.push('/');
                },
              },
            },
          ],
        });
      } else if (filial.value.CDFILIAL === filial.value.CDFILCENTPME) {
        text = 'scm::BLOCK_ACCESS_INVENTORY';
        DialogService.show({
          name: 'blockAccessInventory',
          type: 'error',
          title: I18n.translate('scm::ATTENTION'),
          text: I18n.translate(text),
          buttons: [
            {
              name: 'ok',
              component: 'ZdButton',
              label: I18n.translate('scm::OK'),
              events: {
                click: () => {
                  DialogService.hide();
                  Router.push('/');
                },
              },
            },
          ],
        });
      } else {
        this.checkStock();
      }
    } else {
      this.appController.unidadeLogadaIsValid();
    }
  }

  public checkStock() {
    const ds = DatasourceFactory.factory({
      uniqueKey: 'CDFILIAL',
      type: 'rest',
      route: '/parametros',
      filter: {
        CDFILIAL: localStorage.getItem('CDFILIALLOGADA') || '',
      },
    });
    ds.get().then((result: IDictionary) => {
      let text = '';
      if (result.length && result[0].IDUTILALMOX === 'S') {
        text = 'Esta unidade possui controle de estoque por almoxarifado, impossível lançar inventários!';
      } else if (result.length && result[0].IDUTILLOTE === 'S') {
        text = 'Esta unidade possui controle de estoque por lote, impossível lançar inventários!';
      }
      if (text !== '') {
        DialogService.show({
          name: 'blockAccessInventoryStock',
          type: 'error',
          title: I18n.translate('scm::ATTENTION'),
          text: I18n.translate(text),
          buttons: [
            {
              name: 'ok',
              component: 'ZdButton',
              label: I18n.translate('scm::OK'),
              events: {
                click: () => {
                  DialogService.hide();
                  Router.push('/');
                },
              },
            },
          ],
        });
      }
    });
  }
}
