import {
  Metadata, dayjs as moment, I18n, IEventParam, Loader, IDictionary,
} from '@zeedhi/core';
import {
  IterableComponentRender, Form, Text, Switch, Select, DialogService, LoadingService, Button,
} from '@zeedhi/common';
import { BottomSheet } from '@zeedhi/zd-bottom-sheet';
import { HttpCRUDTranslator } from '@/http-crud-translator/http-crud-translator';

export class SolicitacaoComprasController {
  private utilities = Loader.getInstance('Utilities');

  public showAddProduto = false;

  private periodo = {
    DTINICIAL: '',
    DTFINAL: '',
  };

  get isVisibleDetailList() {
    return !this.showAddProduto;
  }

  public get setFilterSC() {
    return {
      CDFILIAL: localStorage.getItem('CDFILIALLOGADA') || '',
      DTINICIAL: this.periodo.DTINICIAL || '',
      DTFINAL: this.periodo.DTFINAL || '',
    };
    // [
    //   {
    //     name: 'CDFILIAL',
    //     value: localStorage.getItem('CDFILIALLOGADA') || '',
    //     operator: '=',
    //   },
    //   {
    //     name: 'DTINICIAL',
    //     value: dataIni,
    //     operator: '=',
    //   },
    //   {
    //     name: 'DTFINAL',
    //     value: dataFin,
    //     operator: '=',
    //   },
    // ];
  }

  public setValueQtde({ component }: IEventParam<Text>) {
    const value = component.name.split('-').reverse()[0] || '';
    const mask = {
      decimalPlates: 5,
    };
    const maskNumber = this.utilities.getMaskNumber(value, mask);
    component.text = `${I18n.translate('Qtde: ')} ${maskNumber}`;
  }

  public setValueQtdeAjustada({ component }: IEventParam<Text>) {
    const value = component.name.split('-').reverse()[0] || '';
    const mask = {
      decimalPlates: 5,
    };
    const maskNumber = this.utilities.getMaskNumber(value, mask);
    component.text = `${I18n.translate('Qtde Ajustada: ')} ${maskNumber}`;
  }

  public openAddProdutoSC() {
    const form = Metadata.getInstance<Form>('addProdutoSCForm');
    form.clearForm();
    this.showAddProduto = true;
    // this.setNotPossibleDates();
  }

  public clickBackAddProduto() {
    this.showAddProduto = false;
    const cardSC = Metadata.getInstance<IterableComponentRender>('iterableCardsSCompras');
    cardSC.reload();
  }

  public disableJustExtra({ component }: IEventParam<Switch>) {
    const justExtra = Metadata.getInstance<Select>('DSJUSTSOLEXT');
    if (component.value === true) {
      justExtra.isVisible = true;
      justExtra.addValidation('required', { message: 'pme::campoObrigatorio' });
    } else {
      justExtra.isVisible = false;
      justExtra.removeValidation('required');
    }
  }

  public allowedDates(date: string) {
    const today = new Date();
    const inputDate = new Date(date);

    return inputDate >= today;
  }

  public produtoJaExisteParaEssaData(nomeProduto: string, dataDeEntregaProduto: string, codigoProduto: string) {
    const scsRender = Metadata.getInstance<IterableComponentRender>('iterableCardsSCompras');
    const allData = scsRender.datasource.data;

    let existe = false;
    const cdfilsoli = localStorage.getItem('CDFILIALLOGADA');
    const produtoFiltrado = allData.filter((produto: IDictionary) => produto.CDPRODUTO === codigoProduto
      && produto.NMPRODUTO === nomeProduto
      && produto.DTPROGENTR === moment(dataDeEntregaProduto, 'YYYY-MM-DD').format('DD/MM/YYYY')
      && produto.CDFILSOLI === cdfilsoli);

    if (produtoFiltrado.length > 0) existe = true;
    return existe;
  }

  public salvaProdutoSC() {
    const form = Metadata.getInstance<Form>('addProdutoSCForm');
    if (form.validate()) {
      const valueForm = form.value;
      const today = new Date().toJSON().slice(0, 10);
      const todayHour = new Date().toJSON().slice(0, 19).replace('T', ' ');

      if (valueForm.IDITMSOLEXTRA === undefined) valueForm.IDITMSOLEXTRA = 'N';

      const dataset = [
        {
          CDFILLAN: '0001',
          CDFILSOLI: localStorage.getItem('CDFILIALLOGADA') || '',
          CDPRODUTO: valueForm.CDPRODUTO.CDPRODUTO,
          DSJUSTOPER: valueForm.DSJUSTOPER,
          DTPROGENTR: moment(valueForm.DTPROGENTR, 'YYYY-MM-DD').format('DD/MM/YYYY'),
          DTUTILIZA: moment(valueForm.DTPROGENTR, 'YYYY-MM-DD').format('DD/MM/YYYY'),
          NMPRODUTO: valueForm.CDPRODUTO.NMPRODUTO,
          CDFILLANCA: '0001',
          NRSOLICMP: 'A ser criada',
          IDITMSOLEXTRA: valueForm.IDITMSOLEXTRA,
          DSJUSTSOLEXT: valueForm.DSJUSTSOLEXT || null,
          QTAJUSTADA: null,
          QTSOLIPROD: valueForm.QTSOLIPROD,
          ROWNUM: 'L1',
          SGUNIDADE: valueForm.SGUNIDADE,
          IDSOLIAPROV: 'N',
          CDFILIOPELOG: null,
          DTSOLCMP: moment(today, 'YYYY-MM-DD').format('DD/MM/YYYY'),
          IDTIPLAN: 'M',
          IDSOLEXTRA: 'N',
          CDFILCENT: '0001',
          CDOPERLANC: localStorage.getItem('CDOPERADOR'),
          DTHRLANCSOL: moment(todayHour, 'YYYY-MM-DD HH:mm:ss').format('DD/MM/YYYY HH:mm:ss'),
          QTITEMSOLIC: 'Definir na exportação',
          // NRSOLIMOV: 'A ser criada',
          changed: true,
        },
      ];

      const existe = this.produtoJaExisteParaEssaData(valueForm.CDPRODUTO.NMPRODUTO, valueForm.DTPROGENTR,
        valueForm.CDPRODUTO.CDPRODUTO);

      if (existe) {
        this.utilities.showAlert('alertError',
          `${I18n.translate('scm::ROW_EXISTS')}`, 'warning');
      } else {
        // this.criarLogMovopsol(dataset, 'I', '0', valueForm.QTSOLIPROD);
        LoadingService.show(' ');
        HttpCRUDTranslator.post('/produtosSolicitacao/save', { requestType: 'DataSet', dataset })
          .then((response: IDictionary) => {
          // Http.post('/NovaSolicitacao', { requestType: 'DataSet', dataset }).then((response: IDictionary) => {
          //   let solicitacao = response.data;
          //   if (!solicitacao) solicitacao = [];
          //   this.novaSolicitacao(valueForm, solicitacao);
            if (!response.data.error) {
              this.utilities.showAlert('successSave', 'SUCCESS_SAVE', 'green');
              form.clearForm();
            } else {
              const mensagem = response.data.error;
              this.utilities.showAlert('errorSave', I18n.translate(mensagem), 'error');
            }
          }).catch((error) => {
            this.utilities.showAlert('alert', error, 'error');
          }).finally(() => {
            LoadingService.hide();
          });
      }
    }
  }

  public clickDeleteSCompras({ component, event }: IEventParam<Button>) {
    (event as IDictionary).stopPropagation();
    DialogService.show({
      name: 'deleteSCompras',
      type: 'warning',
      title: I18n.translate('scm::ATTENTION'),
      text: I18n.translate('scm::CONFIRM_DELETE_REGISTER'),
      buttons: [
        {
          name: 'yes',
          component: 'ZdButton',
          label: I18n.translate('scm::YES'),
          events: { click: () => this.deleteSCompras({ component }) },
          outline: true,
        },
        {
          name: 'no',
          component: 'ZdButton',
          label: I18n.translate('scm::NO'),
          events: { click: () => DialogService.hide() },
        },
      ],
    });
  }

  public deleteSCompras({ component }: IEventParam<Button>) {
    const cdproduto = component.name.split('-')[1].split('_')[0];
    const dtprogentr = component.name.split('-')[1].split('_')[1];
    const cdfilsoli = component.name.split('-')[1].split('_')[2];
    const cdfillan = component.name.split('-')[1].split('_')[3];
    const nrsolicmp = component.name.split('-')[1].split('_')[4];
    const idsoliaprov = component.name.split('-')[1].split('_')[5];
    const dtFormat = moment(dtprogentr, 'DD/MM/YYYY').format('DDMMYYYY');
    const key = `${cdfilsoli}${cdfillan}${nrsolicmp}${cdproduto}${dtFormat}`;
    const dataset = [
      {
        CDFILIAL: localStorage.getItem('CDFILIALLOGADA') || '',
        CDPRODUTO: cdproduto,
        DTPROGENTR: dtprogentr,
        CDFILSOLI: cdfilsoli,
        CDFILLAN: cdfillan,
        NRSOLICMP: nrsolicmp,
        IDSOLIAPROV: idsoliaprov,
        KEY: key,
      },
    ];
    LoadingService.show(' ');
    HttpCRUDTranslator.post('/produtosSolicitacao/delete', { requestType: 'DataSet', dataset }, {}).then(() => {
      // this.utilities.showAlert('successSave', 'SUCCESS_SAVE', 'green', false);
      const sCompras = Metadata.getInstance<IterableComponentRender>('iterableCardsSCompras');
      sCompras.reload();
    }).catch((error: string) => {
      this.utilities.showAlert('alert', `${error}`, 'error');
    }).then(() => {
      DialogService.hide();
      LoadingService.hide();
    });
  }

  public openFilter() {
    const btn = Metadata.getInstance<Button>('filterSCButton');
    btn.click();
  }

  public clickFilterScompras() {
    const bottomSheet = Metadata.getInstance<BottomSheet>('filterSheet');
    const filial = Metadata.getInstance<Select>('CDFILIALLOGADA');
    if (filial.value) {
      bottomSheet.isVisible = true;
      bottomSheet.isOpen = true;
    }
  }

  public clickCloseFilterBtn() {
    const bottomSheet = Metadata.getInstance<BottomSheet>('filterSheet');
    bottomSheet.isVisible = false;
    bottomSheet.isOpen = false;
  }

  public setPeriodoSC({ component }: IEventParam<Button>) {
    const form = component.parent as Form;
    if (form.validate() && form.value.PERIODO.length === 2) {
      this.periodo.DTINICIAL = moment(form.value.PERIODO[0], 'YYYY-MM-DD').format('DD/MM/YYYY');
      this.periodo.DTFINAL = moment(form.value.PERIODO[1], 'YYYY-MM-DD').format('DD/MM/YYYY');
      const sc = Metadata.getInstance<IterableComponentRender>('iterableCardsSCompras');
      sc.datasource.search = form.value.search;
      sc.datasource.order = [];
      sc.reload();
      const bottomSheet = Metadata.getInstance<BottomSheet>('filterSheet');
      bottomSheet.isVisible = false;
      bottomSheet.isOpen = false;
    }
  }

  // public novaSolicitacao(currentRow: IDictionary, solicitacao: IDictionary) {
  //   const existeSolicitacao = solicitacao.filter((solicita: IDictionary) => solicita.CDFILLANCA === '0001'
  //     && solicita.CDFILSOLI === localStorage.getItem('CDFILIALLOGADA'),
  //   );

  //   if (existeSolicitacao.length === 0) {
  //     const objetoSolicitacao = {
  //       CDFILLANCA: '0001',
  //     };
  //     solicitacao.push(objetoSolicitacao);
  //     return Environment.setDataSource('/NovaSolicitacao', solicitacao);
  //   }
  // }

  // public criarLogMovopsol(currentRow: IDictionary, idoperamovs: string, qtantoper: string, qtdepoper: string) {
  //   const rowNum = 'L1';
  //   const today = new Date().toJSON().slice(0, 10);
  //   // const todayHour = new Date().toJSON().slice(0, 19).replace('T', ' ');
  //   const dataset = [
  //     {
  //       ROWNUM: rowNum,
  //       CDFILSOLMOV: currentRow.CDFILSOLMOV,
  //       CDFILLANMOV: currentRow.CDFILLAN,
  //       NRSOLIMOV: currentRow.NRSOLICMP,
  //       CDPRODUTO: currentRow.CDPRODUTO,
  //       DTPROGENTR: currentRow.DTPROGENTR,
  //       DTOPERSOL: today,
  //       CDOPERADOR: localStorage.getItem('CDOPERADOR'),
  //       DSJUSTOPER: currentRow.DSJUSTOPER || null,
  //       DSJUSTSOLEXT: currentRow.DSJUSTSOLEXT || null,
  //       QTANTOPER: qtantoper,
  //       QTDEPOPER: qtdepoper,
  //       IDOPERAMOVS: idoperamovs,
  //       IDITMSOLEXTRA: currentRow.IDITMSOLEXTRA,
  //       changed: true,
  //     },
  //   ];
  // }
}
