import {
  IDictionary, dayjs as moment, I18n, Config, FormatterParserProvider, Router, IEventParam, Metadata,
} from '@zeedhi/core';
import {
  AlertService, IterableComponentRender, Select,
} from '@zeedhi/common';
import { ScmMask } from '@/config';
import { RequestErrorLog } from '@/server/request-logger';

export class Utilities {
  private listener = false;

  public getItemLocalStorage(itemName: string, key?: string, value?: string) {
    const allRows = JSON.parse(<string>localStorage.getItem(itemName)) || [];
    if (key && value) {
      return allRows.find((el: IDictionary) => el[`${key}`] === value);
    }
    return allRows;
  }

  public setItemLocalStorage(itemName: string, values: object[]) {
    const result = localStorage.getItem(itemName);
    if (result !== null) {
      const obj = JSON.parse(result);
      obj.push(values);
      localStorage.setItem(itemName, JSON.stringify(obj));
    } else {
      localStorage.setItem(itemName, JSON.stringify(values));
    }
  }
  /*
  public getIDBItem(elementName: string, raw?: boolean) {
    new Promise((resolve) => {
      localforage.getItem(elementName, () => {}).then((item: any) => {
        let result;
        if (!raw) {
          result = JSON.parse(item);
        } else result = item;
        resolve(result);
      });
    });
  }

  public setIDBItem(elementName: string, content: object[], raw?: boolean) {
    let result;
    if (!raw) {
      result = JSON.stringify(content);
    } else result = content;
    return localforage.setItem(elementName, result);
  }
  */

  public validateInterval(ini: string, fim: string) {
    if (ini && fim) {
      const iniValue = moment(ini, 'DD/MM/YYYY');
      const fimValue = moment(fim, 'DD/MM/YYYY');
      if (fimValue <= iniValue) {
        this.showAlert('invalidInterval', 'SELECT_COMPLETE_INTERVAL', 'error');
        return false;
      }
      // if (fimValue.diff(iniValue, 'days') === 0) {
      //   this.showAlert('invalidInterval', 'SELECT_COMPLETE_INTERVAL', 'error');
      //   return false;
      // }
      if (fimValue.diff(iniValue, 'days') > 31) {
        this.showAlert('intervalTooBig', 'SELECT_31_DAYS_INTERVAL', 'error');
        return false;
      }

      return true;
    }
    this.showAlert('invalidInterval', 'SELECT_COMPLETE_INTERVAL', 'error');
    return false;
  }

  public spliceOnCondition(array: object[], condition: (el: any, index: number) => boolean) {
    let i = array.length - 1;
    while (i) {
      if (condition(array[i], i)) {
        array.splice(i, 1);
      }
      i -= 1;
    }
  }

  get getUnidadeLogada() {
    const cdfilial = localStorage.getItem('CDFILIALLOGADA');
    const nmfilial = localStorage.getItem('NMFILIALLOGADA');
    return `${cdfilial} - ${nmfilial}`;
  }

  public showAlert(name: string, error: any, color = 'green', top = true, timeout = 3000) {
    const text = error?.response?.data;
    let msg;

    if (text) {
      msg = I18n.translate(`${text}`);
    } else {
      msg = I18n.translate(`scm::${error}`);
    }

    const alertType = {
      name,
      color,
      text: msg,
      top,
      left: false,
      right: true,
      timeout,
      buttonsSlot: [
        {
          name: 'undo-button',
          component: 'ZdButton',
          icon: true,
          iconName: 'close',
          color: 'white',
          events: {
            click: () => { AlertService.hide(); },
          },
        },
      ],
    };
    AlertService.show(alertType);
  }

  public getLanguage() {
    return localStorage.getItem('LOGIN_LANGUAGE') || 'pt_br';
  }

  public getFormatLocale(): string {
    return (localStorage.getItem('LOGIN_LANGUAGE') !== 'en_us') ? 'DD/MM/YYYY' : 'MM/DD/YYYY';
  }

  public getMaskNumber(value: string, mask: object[]) {
    const formatterFn = FormatterParserProvider.getFormatter('ZdNumber');
    const decimalMask = `${formatterFn(value, mask)}`;
    return decimalMask;
  }

  public formatDate(date: string) {
    return moment(date, 'DD/MM/YYYY').format(Config.displayFormat);
  }

  public getMaskProduto(value: string) {
    const maskPattern: string = (Config.masks.productMask as ScmMask).pattern;
    const totalValue = value.length;
    let maskReturn = '';
    for (let i = 0, insertedNumbers = 0; insertedNumbers !== totalValue; i += 1) {
      if (maskPattern.charAt(i) === '.') {
        maskReturn = `${maskReturn}.`;
      } else {
        maskReturn = `${maskReturn}N`;
        insertedNumbers += 1;
      }
    }
    return maskReturn;
  }

  public getValueWithMaskProduto(value: string) {
    const maskPattern: string = (Config.masks.productMask as ScmMask).pattern;
    const totalValue = value.length;
    let maskReturn = '';
    for (let i = 0, insertedNumbers = 0; insertedNumbers !== totalValue; i += 1) {
      if (maskPattern.charAt(i) === '.') {
        maskReturn = `${maskReturn}.`;
      } else {
        maskReturn = `${maskReturn}${value.charAt(insertedNumbers)}`;
        insertedNumbers += 1;
      }
    }
    return maskReturn;
  }

  public get nmOperador() {
    return ` ${localStorage.getItem('LOGIN_USER_NAME')}` || ' ';
  }

  public setEventLoadMoreIterable({ component }: IEventParam<IterableComponentRender>) {
    if (!this.listener) {
      document.addEventListener('scroll', (event: any) => {
        const yPosition = window.scrollY;
        if (window.scrollY + window.outerHeight + 110 > document.body.scrollHeight - 200) {
          const { limit, total, loading } = component.datasource;
          if (limit < total && !loading) {
            component.datasource.limit = limit + 10;
            component.reload().then(() => {
              window.scrollTo(0, yPosition);
            });
          }
        }
        event.stopPropagation();
      });
      this.listener = true;
    }
  }

  public openErrors() {
    Router.push('/errors');
    localStorage.setItem('notSeenErrorsCount', '0');
  }

  public get checkUnitySelected() {
    const filial = Metadata.getInstance<Select>('CDFILIALLOGADA');
    return !filial.value;
  }

  public get errorsLogData() {
    const item = localStorage.getItem('request-error-log') || '[]';
    const errorLogs: RequestErrorLog[] = JSON.parse(item);
    return errorLogs;
  }

  public get checkOnline() {
    return !navigator.onLine;
  }
}
