import { IDictionary, IHttpResponse, Interceptor } from '@zeedhi/core';

export interface RequestLog {
  method: 'put' | 'post' | 'delete';
  url: string;
  row: IDictionary;
  operation: string;
  timestamp: number;
  route: string;
}

export interface RequestErrorLog {
  method: 'put' | 'post' | 'delete';
  url: string;
  row: IDictionary;
  operation: string;
  timestamp: number;
  error: string;
}

const registerLogger = () => {
  Interceptor.addResponseSuccess((response: IHttpResponse<{ data: IDictionary }>) => {
    // se estiver online não precisamos salvar no histórico
    if (navigator.onLine) return response;

    // const { statusText } = response;
    const { method } = response.config;
    const url = response.config.url || '';

    // cria um log ao adicionar/modificar/deletar um recurso
    if (method === 'post' || method === 'put' || method === 'delete') {
      const timestamp = Date.now();
      let operation = '';
      switch (method) {
        case 'post':
          operation = 'create';
          break;
        case 'put':
          operation = 'update';
          break;
        case 'delete':
          operation = 'delete';
          break;
        default:
          break;
      }
      const route = url.split('/')[0];
      let row = response.data.data;
      if (method === 'put' || method === 'delete') {
        row = JSON.parse(response.config.data);
      }
      const newLog: RequestLog = {
        method, url, row, operation, timestamp, route,
      };
      const item = localStorage.getItem('request-log') || '[]';
      const logs: RequestLog[] = JSON.parse(item);

      logs.push(newLog);

      // salva o log na localstorage
      const stringified = JSON.stringify(logs);
      localStorage.setItem('request-log', stringified);
    }

    return response;
  });
};

const registerErrorLogger = (response: IDictionary) => {
  const item = localStorage.getItem('request-error-log') || '[]';
  const errorLogs: RequestErrorLog[] = JSON.parse(item);

  const statusText = response?.statusText || '';
  const { method } = response?.config;
  const url = response?.config?.url || '';
  const row = response?.data?.data;
  const timestamp = Date.now();
  const operation = statusText?.slice(0, -1).toLowerCase();
  const { error } = response?.data;
  const newLog: RequestErrorLog = {
    method, url, row, operation, timestamp, error,
  };
  errorLogs.push(newLog);

  // salva o log na localstorage
  const stringified = JSON.stringify(errorLogs);
  localStorage.setItem('request-error-log', stringified);
};

export { registerLogger, registerErrorLogger };
