import { makeServer, clearStorage } from '@zeedhi/zeedhi-miragejs';
import {
  Interceptor, Http, IDictionary, JsonCacheService,
} from '@zeedhi/core';
import adapter from '@zeedhi/axios-mirage-adapter';
import { DialogService, LoadingService } from '@zeedhi/common';
import { cacheMetadata } from '@/router';
import { useMirage } from './use-mirage';
import { registerLogger, RequestLog, registerErrorLogger } from './request-logger';
import { getConfig } from './config';

Interceptor.addRequestSuccess((axiosConfig) => ({ ...axiosConfig, adapter }));

const endpoint = process.env.VUE_APP_MIRAGE_END_POINT;
let server = makeServer({
  endpoint, config: getConfig(), logging: true, persist: true,
});
const callMakeServer = (serverConfig: IDictionary) => {
  const p = new Promise((resolve) => {
    clearStorage();
    server.shutdown();
    server = makeServer({
      endpoint, config: serverConfig, logging: true, persist: true,
    });
    // useMirage(true);
    // Http.delete('/inventarios/01000000000176').then((result) => {
    //   console.warn(result);
    // }).catch((error) => {
    //   console.warn(error);
    // });
    // useMirage(false);
    // return server;
    resolve(server);
  });
  return p;
};

const executeExport = () => {
  const item = localStorage.getItem('request-log') || '[]';
  const logs = JSON.parse(item);
  LoadingService.show(' ');
  // para cada item do histórico, realiza uma request para o backend
  const pLogs = logs.map(({
    method, row, operation, route,
  }: RequestLog) => {
    if (method === 'post' || method === 'delete' || method === 'put') {
      // Http.post('/manufacturers', row);
      if (operation === 'create') {
        return Http.post(`${route}/save`, { requestType: 'DataSet', dataset: [row] })
          .then((response: IDictionary) => {
            if (response.data?.length !== 0) {
              // const mensagem = response.data.error;
              registerErrorLogger(response);
            }
          }).catch((error) => {
            registerErrorLogger(error.response);
          });
      }
      if (operation === 'delete') {
        return Http.post(`${route}/delete`, { requestType: 'DataSet', dataset: [row] })
          .then((response: IDictionary) => {
            if (response.data?.length !== 0) {
              registerErrorLogger(response);
            }
          }).catch((error) => {
            registerErrorLogger(error.response);
          });
      }
      if (operation === 'update') {
        return Http.post(`${route}/update`, { requestType: 'DataSet', dataset: [row] })
          .then((response: IDictionary) => {
            if (response.data?.length !== 0) {
              // const mensagem = response.data.error;
              registerErrorLogger(response);
            }
          }).catch((error) => {
            registerErrorLogger(error.response);
          });
      }
    }
    return null;
  });
  return Promise.all(pLogs).then(async () => {
    JsonCacheService.clearJSONCache(cacheMetadata);
    JsonCacheService.saveJSONCache(cacheMetadata);
    // await reload();
    await clearStorage();
    // limpa o histórico
    localStorage.removeItem('request-log');
    localStorage.setItem('hasImportedData', 'false');
    LoadingService.hide();
    DialogService.hide();
  });
};

if (!navigator.onLine) {
  useMirage();
}

registerLogger();

window.addEventListener('online', async () => {
  useMirage(false);
  // const hasImportedData = localStorage.getItem('hasImportedData');
  // const item = localStorage.getItem('request-log') || '[]';
  // const logs = JSON.parse(item);
  // if (hasImportedData === 'true' && logs.length > 0) {
  //   DialogService.show({
  //     name: 'askForExport',
  //     type: 'warning',
  //     title: I18n.translate('scm::ATTENTION'),
  //     text: I18n.translate('scm::ONLINE_EXPORT'),
  //     buttons: [
  //       {
  //         name: 'yes',
  //         component: 'ZdButton',
  //         label: I18n.translate('scm::YES'),
  //         events: {
  //           click: () => {
  //             executeExport();
  //           },
  //         },
  //         outline: true,
  //       },
  //       {
  //         name: 'no',
  //         component: 'ZdButton',
  //         label: I18n.translate('scm::NO'),
  //         events: { click: () => DialogService.hide() },
  //       },
  //     ],
  //   });
  // }
});

window.addEventListener('offline', async () => {
  useMirage();
});

export { callMakeServer, executeExport };
